@@ -17,20 +17,11 @@ fi
 
 CMAKE_FLAGS="-DCMAKE_INSTALL_PREFIX=$INSTALL"
 
-# setting the rpath so that libOpenMMPME.so finds the right libfftw3
-#CMAKE_FLAGS+=" -DCMAKE_INSTALL_RPATH=.."
 CMAKE_FLAGS+=" -DCMAKE_C_COMPILER=clang -DCMAKE_CXX_COMPILER=clang++"
 CMAKE_FLAGS+=" -DCMAKE_OSX_DEPLOYMENT_TARGET=10.9"
 CMAKE_FLAGS+=" -DCMAKE_OSX_SYSROOT=/Applications/Xcode.app/Contents/Developer/Platforms/MacOSX.platform/Developer/SDKs/MacOSX10.11.sdk"
 CMAKE_FLAGS+=" -DOPENMM_GENERATE_API_DOCS=ON"
 
-# Build in subdirectory.
-# Set location for FFTW3
-PREFIX="$WORKSPACE/miniconda"
-CMAKE_FLAGS+=" -DFFTW_INCLUDES=$PREFIX/include"
-CMAKE_FLAGS+=" -DFFTW_LIBRARY=$PREFIX/lib/libfftw3f.dylib"
-CMAKE_FLAGS+=" -DFFTW_THREADS_LIBRARY=$PREFIX/lib/libfftw3f_threads.dylib"
-
 # Build in subdirectory.
 if [ -e build ]; then
     rm -rf build
