@@ -64,6 +64,11 @@ html_theme_options = {
             "uri": "../api-c++/",
             "relative": True,
         },
+        {
+            "title": "Cookbook & Tutorials",
+            "uri": "https://openmm.github.io/openmm-cookbook/",
+            "relative": False,
+        },
         {
             "title": "GitHub",
             "uri": "https://github.com/openmm",
