@@ -44,6 +44,7 @@ using std::string;
 using std::stringstream;
 
 CustomBondForceImpl::CustomBondForceImpl(const CustomBondForce& owner) : owner(owner) {
+    forceGroup = owner.getForceGroup();
 }
 
 CustomBondForceImpl::~CustomBondForceImpl() {
@@ -79,7 +80,7 @@ void CustomBondForceImpl::initialize(ContextImpl& context) {
 }
 
 double CustomBondForceImpl::calcForcesAndEnergy(ContextImpl& context, bool includeForces, bool includeEnergy, int groups) {
-    if ((groups&(1<<owner.getForceGroup())) != 0)
+    if ((groups&(1<<forceGroup)) != 0)
         return kernel.getAs<CalcCustomBondForceKernel>().execute(context, includeForces, includeEnergy);
     return 0.0;
 }
