@@ -68,6 +68,7 @@ public:
 };
 
 CustomCentroidBondForceImpl::CustomCentroidBondForceImpl(const CustomCentroidBondForce& owner) : owner(owner) {
+    forceGroup = owner.getForceGroup();
 }
 
 CustomCentroidBondForceImpl::~CustomCentroidBondForceImpl() {
@@ -120,7 +121,7 @@ void CustomCentroidBondForceImpl::initialize(ContextImpl& context) {
 }
 
 double CustomCentroidBondForceImpl::calcForcesAndEnergy(ContextImpl& context, bool includeForces, bool includeEnergy, int groups) {
-    if ((groups&(1<<owner.getForceGroup())) != 0)
+    if ((groups&(1<<forceGroup)) != 0)
         return kernel.getAs<CalcCustomCentroidBondForceKernel>().execute(context, includeForces, includeEnergy);
     return 0.0;
 }
