@@ -43,6 +43,7 @@ using namespace OpenMM;
 using namespace std;
 
 HarmonicAngleForceImpl::HarmonicAngleForceImpl(const HarmonicAngleForce& owner) : owner(owner) {
+    forceGroup = owner.getForceGroup();
 }
 
 HarmonicAngleForceImpl::~HarmonicAngleForceImpl() {
@@ -70,7 +71,7 @@ void HarmonicAngleForceImpl::initialize(ContextImpl& context) {
 }
 
 double HarmonicAngleForceImpl::calcForcesAndEnergy(ContextImpl& context, bool includeForces, bool includeEnergy, int groups) {
-    if ((groups&(1<<owner.getForceGroup())) != 0)
+    if ((groups&(1<<forceGroup)) != 0)
         return kernel.getAs<CalcHarmonicAngleForceKernel>().execute(context, includeForces, includeEnergy);
     return 0.0;
 }
