@@ -39,6 +39,7 @@ using namespace OpenMM;
 using namespace std;
 
 HarmonicBondForceImpl::HarmonicBondForceImpl(const HarmonicBondForce& owner) : owner(owner) {
+    forceGroup = owner.getForceGroup();
 }
 
 HarmonicBondForceImpl::~HarmonicBondForceImpl() {
@@ -66,7 +67,7 @@ void HarmonicBondForceImpl::initialize(ContextImpl& context) {
 }
 
 double HarmonicBondForceImpl::calcForcesAndEnergy(ContextImpl& context, bool includeForces, bool includeEnergy, int groups) {
-    if ((groups&(1<<owner.getForceGroup())) != 0)
+    if ((groups&(1<<forceGroup)) != 0)
         return kernel.getAs<CalcHarmonicBondForceKernel>().execute(context, includeForces, includeEnergy);
     return 0.0;
 }
