@@ -230,7 +230,9 @@ class Topology(object):
                     yield atom
 
     def bonds(self):
-        """Iterate over all bonds (each represented as a tuple of two Atoms) in the Topology."""
+        """Iterate over all bonds in the Topology.
+        Each one is represented by a Bond object, which is a named tuple of two atoms.
+        """
         return iter(self._bonds)
 
     def getPeriodicBoxVectors(self):
