@@ -96,7 +96,7 @@ def getText(subNodePath, node):
             s = "%s\n\n" % s
     return s.strip()
 
-OPENMM_RE_PATTERN=re.compile("(.*)OpenMM:[a-zA-Z:]*:(.*)")
+OPENMM_RE_PATTERN=re.compile("(.*)OpenMM:[a-zA-Z0-9:]*:(.*)")
 def stripOpenmmPrefix(name, rePattern=OPENMM_RE_PATTERN):
     try:
         m=rePattern.search(name)
@@ -261,10 +261,12 @@ class SwigInputBuilder:
                     baseName = getText("compoundname", baseNode)
                     if baseName == 'OpenMM::Force':
                         forceSubclassList.append(shortClassName)
-                    elif baseName == 'OpenMM::Integrator':
+                    elif baseName in ('OpenMM::Integrator', 'OpenMM::DrudeIntegrator'):
                         integratorSubclassList.append(shortClassName)
                     elif baseName == 'OpenMM::TabulatedFunction':
                         tabulatedFunctionSubclassList.append(shortClassName)
+        # We need to include subclasses of DrudeIntegrator, but not DrudeIntegrator itself.
+        integratorSubclassList.remove('DrudeIntegrator')
 
         self.fOut.write("%factory(OpenMM::Force& OpenMM::System::getForce")
         for name in sorted(forceSubclassList):
@@ -711,6 +713,7 @@ class SwigInputBuilder:
         self.fOut.write("/* Swig input file,\n")
         self.fOut.write("%sgenerated by %s on %s\n*/\n\n\n"
                         % (INDENT, sys.argv[0], time.asctime()))
+        self.fOut.write("%include \"factory.i\"\n")
         self.fOut.write("\nnamespace OpenMM {\n\n")
         self.writeFactories()
         self.writeGlobalConstants()
