@@ -137,7 +137,7 @@ class TestIntegrators(unittest.TestCase):
                 force.setForceGroup(0)
 
         # Create an integrator
-        integrator = MTSLangevinIntegrator(300*kelvin, 1/picosecond, 4*femtoseconds, [(2,1), (1,2), (0,4)])
+        integrator = MTSLangevinIntegrator(300*kelvin, 5/picosecond, 4*femtoseconds, [(2,1), (1,2), (0,4)])
 
         # Run some equilibration.
         context = Context(system, integrator)
@@ -156,13 +156,43 @@ class TestIntegrators(unittest.TestCase):
         temperature = averageEnergy*2/(dofs*MOLAR_GAS_CONSTANT_R)
         self.assertTrue(290*kelvin < temperature < 310*kelvin)
 
+    def testMTSLangevinIntegratorFriction(self):
+        """Test the MTSLangevinIntegrator on a force-free particle to ensure friction is properly accounted for (issue #3790)"""
+        # Create a System with a single particle and no forces
+        system = System()
+        system.addParticle(12.0*amu)
+        platform = Platform.getPlatformByName('Reference')
+        initial_positions = [Vec3(0,0,0)]
+        initial_velocities = [Vec3(1,0,0)]
+        nsteps = 125 # number of steps to take
+        collision_rate = 1/picosecond
+        timestep = 4*femtoseconds
+
+        def get_final_velocities(nsubsteps):
+            """Get the final velocity vector after a fixed number of steps for the specified number of substeps"""
+            integrator = MTSLangevinIntegrator(0*kelvin, collision_rate, timestep, [(0,nsubsteps)])
+            context = Context(system, integrator, platform)
+            context.setPositions(initial_positions)
+            context.setVelocities(initial_velocities)
+            integrator.step(nsteps)
+            final_velocities = context.getState(getVelocities=True).getVelocities()
+            del context, integrator
+            return final_velocities
+
+        # Compare sub-stepped MTS with single-step MTS
+        for nsubsteps in range(2,6):
+            mts_velocities = get_final_velocities(nsubsteps)
+            self.assertAlmostEqual(math.exp(-timestep*nsteps*collision_rate), mts_velocities[0].x)
+            self.assertAlmostEqual(0, mts_velocities[0].y)
+            self.assertAlmostEqual(0, mts_velocities[0].z)
+
     def testNoseHooverIntegrator(self):
         """Test partial thermostating in the NoseHooverIntegrator (only API)"""
         pdb = PDBFile('systems/alanine-dipeptide-explicit.pdb')
         ff = ForceField('amber99sbildn.xml', 'tip3p.xml')
         system = ff.createSystem(pdb.topology, nonbondedMethod=PME)
 
-        integrator = NoseHooverIntegrator(1.0*femtosecond) 
+        integrator = NoseHooverIntegrator(1.0*femtosecond)
         integrator.addSubsystemThermostat(list(range(5)), [], 200*kelvin, 1/picosecond, 200*kelvin, 1/picosecond, 3,3,3)
         con = Context(system, integrator)
         con.setPositions(pdb.positions)
